"""module for dataframe optimizations"""
import numpy as np
import pandas as pd


# pylint: disable-next=too-many-branches
def df_optimize(df: pd.DataFrame, object_option: bool = False)\
        -> pd.DataFrame:
    """Reduce the size of the input dataframe
    Parameters
    ----------
    df: pd.DataFrame
        input DataFrame
    object_option : bool, default=False
        if true, try to convert object to category
    Returns
    -------
    df: pd.DataFrame
        data type optimized output dataframe
    """

    # loop columns in the dataframe to downcast the dtype
    for col in df.columns:
        # process the int columns
        if df[col].dtype == 'int64':
            col_min = df[col].min()
            col_max = df[col].max()
            # if all are non-negative, change to uint
            if col_min >= 0:
                if col_max < np.iinfo(np.uint8).max:
                    df[col] = df[col].astype(np.uint8)
                elif col_max < np.iinfo(np.uint16).max:
                    df[col] = df[col].astype(np.uint16)
                elif col_max < np.iinfo(np.uint32).max:
                    df[col] = df[col].astype(np.uint32)
                else:
                    df[col] = df[col]
            else:
                # if it has negative values, downcast based on the min and max
                if col_max < np.iinfo(np.int8).max and col_min > np.iinfo(
                        np.int8).min:
                    df[col] = df[col].astype(np.int8)
                elif col_max < np.iinfo(np.int16).max and col_min > np.iinfo(
                        np.int16).min:
                    df[col] = df[col].astype(np.int16)
                elif col_max < np.iinfo(np.int32).max and col_min > np.iinfo(
                        np.int32).min:
                    df[col] = df[col].astype(np.int32)
                else:
                    df[col] = df[col]

        # process the float columns
        elif df[col].dtype == 'float':
            col_min = df[col].min()
            col_max = df[col].max()
            # downcast based on the min and max
            if col_min > np.finfo(np.float32).min and col_max < np.finfo(
                    np.float32).max:
                df[col] = df[col].astype(np.float32)
            else:
                df[col] = df[col]

        if object_option:
            if df[col].dtype == 'object':
                if len(df[col].value_counts()) < 0.5 * len(df):
                    df[col] = df[col].astype('category')

    return df



"""Caching decorator for functions returning pd.DataFrame objects"""
import datetime as dt
import os
from typing import Any

from collections.abc import Callable
from functools import wraps

import pandas as pd


def df_cache(func: Callable[..., pd.DataFrame]) -> Callable[..., pd.DataFrame]:
    """Caching decorator for functions returning pd.DataFrame"""
    cache_dir = 'cache/'

    @wraps(func)
    def inner_func(*args: Any, **kwargs: Any):
        if not args and not kwargs:
            kwargs['runtime'] = dt.date.today()
        file_name = (
            cache_dir
            + func.__name__
            + '_'
            + '_'.join(map(str, args))
            + '_'
            + '_'.join([f'{str(k)}__{str(v)}' for k, v in kwargs.items()])
            + '.snappy.parquet'
        )
        if os.path.isfile(file_name):
            return pd.read_parquet(file_name)

        result = func(*args, **kwargs)
        if not os.path.exists(cache_dir):
            os.makedirs(cache_dir)
        result.to_parquet(file_name, compression='snappy')
        return result

    return inner_func



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['stefan_utils']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.22.3,<2.0.0',
 'pandas>=1.4.1,<2.0.0',
 'pyarrow>=7.0.0,<8.0.0',
 'python-json-logger>=2.0.2,<3.0.0']

setup_kwargs = {
    'name': 'stefan-utils',
    'version': '0.0.2',
    'description': '',
    'long_description': None,
    'author': 'Stefan Bader',
    'author_email': 'baderstefan@gmx.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



"""Module for a json formatted logger"""

import logging

import datetime as dt

from pythonjsonlogger import jsonlogger


def get_logger(name: str) -> logging.Logger:
    """Returns logger instance"""
    logging.captureWarnings(True)
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    if not logger.hasHandlers():
        handler = logging.StreamHandler()
        handler.setFormatter(
            logging.Formatter(
                '[%(asctime)s] %(threadName)s %(name)s'
                ' [%(levelname)7s] %(message)s'
            )
        )
        logger.addHandler(handler)
    return logger


def get_logger_json(name: str = 'logger'):
    """Function to get a new logger instance"""
    log = logging.getLogger(name)
    log.setLevel(logging.INFO)
    if not log.handlers:
        log.addHandler(get_app_logging_handler())
    return log


def get_app_logging_handler() -> logging.StreamHandler:
    """Getting a new logging handler"""
    log_handler = logging.StreamHandler()
    formatter = JsonLogFormatter()
    log_handler.setFormatter(formatter)
    return log_handler


class JsonLogFormatter(jsonlogger.JsonFormatter):
    """Json Log formatter following the Zooplus standards."""

    def add_fields(
            self,
            log_record: dict,
            record: logging.LogRecord,
            message_dict: dict) -> None:
        super().add_fields(log_record, record, message_dict)

        log_record['log_type'] = 'application_log'
        log_record['@timestamp'] = dt.datetime.now(dt.timezone.utc).isoformat()
        log_record['description'] = log_record.pop('message', None)
        log_record['severity'] = record.levelname.upper()
        log_record['class'] = ':'.join(
            [record.module, record.funcName, str(record.lineno)]
        )

        if exc_info := log_record.pop('exc_info', None):
            log_record['stacktrace'] = exc_info
        if record.process:
            log_record['pid'] = record.process
        if record.threadName:
            log_record['thread'] = record.threadName
